#!/bin/ksh

usage="dante [-h] [-d] [-e] [-q N] [number-field-def]"

if [[ $# -eq 0 ]]
then
    print -u2 "usage: ${usage}"
    exit 1
fi

while getopts hdeq: o
do
    case $o in
        h) echo "$usage"; exit 1;;
	d) debug=1;;
	e) emit_instructions=1;;
	q) square_free_integer=$OPTARG;;
    esac
done

shift $((OPTIND-1))

if stty -a >/dev/null 2>&1
then
    interactive=true
else
    interactive=false
fi

number_field_definition=$1

if [[ -z $square_free_integer ]]
then
    if [[ -z $number_field_definition ]]
    then
	print -u2 "error: no number field definition provided"
	exit 1
    fi

    if [[ ! -e $number_field_definition ]]
    then
	print -u2 "error: cannot read number field definition"
	exit 1
    fi

    degree=$( head -n 1 ${number_field_definition} | sed -e 's/].*$/]/' | awk -F, '{print NF}' )

    prompt=${number_field_definition%%.*}

    initialization=$(cat ${number_field_definition})
else
    if [[ -n $number_field_definition ]]
    then
	print -u2 "error: cannot specify both -q and number field definition"
	exit 1
    fi

    degree=2

    prompt="Q(sqrt($square_free_integer))"

    if ! initialization=$(gen-qf.bash $square_free_integer)
    then
	echo "${initialization}"
	exit 1
    fi
fi

slot_base=0
[[ 4 -gt $slot_base ]] && slot_base=4
[[ $((2+2*degree)) -gt $slot_base ]] && slot_base=$((2+2*degree))
[[ $((3+degree**2)) -gt $slot_base ]] && slot_base=$((3+degree**2))

{
    echo "${initialization}"
    [[ $interactive == true  ]] && echo "interactive"

    while read
    do
	[[ $REPLY == @(quit) ]] && break
	print -r -- "$REPLY"
    done
} |

{
    cat base-field.bc polynomial.bc norm.bc discriminant.bc

    auto-gen.bash $degree $prompt

    front-end.bash -b $slot_base |&

    while read
    do
	expr=$REPLY

	passes=0

	while true
	do
	    print -p -- ${expr}
	    [[ -n $debug && -n $enable_debug ]] && print "SENT TO AWK: <${expr}>" > /dev/tty

	    unset expr

	    while read -pr result
	    do
		[[ -n $debug && -n $enable_debug ]] && print "READ FROM AWK: <${result}>" > /dev/tty
		[[ $result == @(EXPR:*)  ]] && expr=${result}
		[[ $result == @(INSTR:*) ]] && print -r "${result#INSTR:}"

                [[ $result == @(INSTR:interactive = 1) ]] && going_interactive=1
                [[ $result == @(DONE:) && -n $going_interactive ]] && enable_debug=1

		[[ $result == @(DONE:*)  ]] && break
	    done

	    [[ -z $expr ]] && break

	    expr=${expr#EXPR:}
            passes=$((passes+1))
            [[ $(( passes % 100 )) -eq 0 ]] && print "AWK PASS: $passes" > /dev/tty
	done

    done
} |

{
    if [[ -n $emit_instructions ]]
    then
	cat
    else
	bc -q
    fi
}
